<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://www.welaunch.io
 * @since      1.0.0
 *
 * @package    WooCommerce_Attribute_Images
 * @subpackage WooCommerce_Attribute_Images/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WooCommerce_Attribute_Images
 * @subpackage WooCommerce_Attribute_Images/public
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WooCommerce_Attribute_Images_Public extends WooCommerce_Attribute_Images {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	protected $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of this plugin.
	 */
	protected $version;

	/**
	 * options of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $options
	 */
	protected $options;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() 
	{
		if($this->get_option('performanceOnlyWooPages') && (!is_shop() && !is_product() && !is_product_category() && !is_product_tag() && !is_product_taxonomy()) ){
			return;
		}

		wp_enqueue_style( $this->plugin_name.'-public', plugin_dir_url( __FILE__ ) . 'css/woocommerce-attribute-images-public.css', array(), $this->version, 'all' );

		$css = "";

		$attributeNameImageWidth = $this->get_option('attributeNameImageWidth');
		$attributeNameImageBorderRadius = $this->get_option('attributeNameImageBorderRadius');

		$css .= "	th .woocommerce-attribute-image {
						max-width: " . $attributeNameImageWidth . "px;
					}
					th .woocommerce-attribute-image img {
						border-radius: " . $attributeNameImageBorderRadius . "%;
					}";

		$attributeImageWidth = $this->get_option('attributeImageWidth');
		$attributeImageBorderRadius = $this->get_option('attributeImageBorderRadius');
		$attributeImageBorderColor = $this->get_option('attributeImageBorderColor');
		$attributeImageActiveBorderColor = $this->get_option('attributeImageActiveBorderColor');

		$categoryVariationSwatchesImageWidth = $this->get_option('categoryVariationSwatchesImageWidth');
		$categoryAttributeImagesImageWidth = $this->get_option('categoryAttributeImagesImageWidth');
		$singleProductAttributeImagesImageWidth = $this->get_option('singleProductAttributeImagesImageWidth');

		$css .= "	.woocommerce-attribute-image {
						max-width: " . $attributeImageWidth . "px;
					}
					.woocommerce-attribute-image img {
						border-radius: " . $attributeImageBorderRadius . "%;
						border-color: " . $attributeImageBorderColor . ";
					}
					.woocommerce-attribute-images-variable-option-active img {
						border-color: " . $attributeImageActiveBorderColor . ";
					}
					.woocommerce-attribute-images-category-variation-swatches .woocommerce-attribute-image {
						max-width: " . $categoryVariationSwatchesImageWidth . "px;
					}
					.woocommerce-attribute-images-category-attribute-images .woocommerce-attribute-image {
						max-width: " . $categoryAttributeImagesImageWidth . "px;
					}
					.woocommerce-attribute-images-single-product-attribute-images .woocommerce-attribute-image {
						max-width: " . $singleProductAttributeImagesImageWidth . "px;
					}

					.woocommerce-attribute-image-color {
						width: " . $attributeImageWidth . "px;
						height: " . $attributeImageWidth . "px;
						border-radius: " . $attributeImageBorderRadius . "%;
						border-color: " . $attributeImageBorderColor . ";
					}
					.woocommerce-attribute-images-variable-option-active .woocommerce-attribute-image-color {
						border-color: " . $attributeImageActiveBorderColor . ";
					}
					
					.woocommerce-attribute-images-single-product-attribute-images .woocommerce-attribute-image-color {
						width: " . $singleProductAttributeImagesImageWidth . "px;
						height: " . $singleProductAttributeImagesImageWidth . "px;
					}

					.woocommerce-attribute-images-category-variation-swatches .woocommerce-attribute-image-color {
						width: " . $categoryVariationSwatchesImageWidth . "px;
						height: " . $categoryVariationSwatchesImageWidth . "px;
					}

					.woocommerce-attribute-images-category-attribute-images .woocommerce-attribute-image-color {
						width: " . $categoryAttributeImagesImageWidth . "px;
						height: " . $categoryAttributeImagesImageWidth . "px;
					}
					";

		file_put_contents(dirname(__FILE__)  . '/css/woocommerce-attribute-images-custom.css', $css);

		wp_enqueue_style($this->plugin_name.'-custom', plugin_dir_url(__FILE__).'css/woocommerce-attribute-images-custom.css', array(), $this->version, 'all');

		return true;
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    http://www.welaunch.io
	 * @return  boolean
	 */
	public function enqueue_scripts()
	{
		if($this->get_option('performanceOnlyWooPages') && (!is_shop() && !is_product() && !is_product_category() && !is_product_tag() && !is_product_taxonomy()) ){
			return;
		}

		wp_enqueue_script($this->plugin_name.'-public', plugin_dir_url(__FILE__).'js/woocommerce-attribute-images-public.js', array('jquery'), $this->version, true);
        $forJS['ajax_url'] = admin_url('admin-ajax.php');
        $forJS['categoryVariationSwatchesImageSelector'] = $this->get_option('categoryVariationSwatchesImageSelector');
        $forJS['categoryVariationSwatchesLinkSelector'] = $this->get_option('categoryVariationSwatchesLinkSelector');
		$forJS['categoryVariationSwatchesPriceSelector'] = $this->get_option('categoryVariationSwatchesPriceSelector');
    	$forJS = apply_filters('woocommerce_attribute_images_options', $forJS);
        wp_localize_script($this->plugin_name . '-public', 'woocommerce_attribute_images_options', $forJS);
	}
	
	/**
	 * Inits Group Attributes
	 *
	 * @since    1.0.0
	 */
    public function init()
    {
		global $woocommerce_attribute_images_options;

		$this->options = $woocommerce_attribute_images_options;

		if(!$this->get_option('enable')) {
			return false;
		}

		$this->term_data = get_option( 'woocommerce_attribute_image_term_data' );
		
		add_filter( 'wc_get_template', array($this, 'modify_attribute_template'), 10, 5 );
		add_filter( 'woocommerce_attribute_value_image', array($this, 'checkAttributeImage'), 10, 3);
		add_filter( 'woocommerce_attribute_name_image', array($this, 'checkAttributeNameImage'), 10, 2);
		add_filter( 'woocommerce_attribute_value_description', array($this, 'checkAttributeDescription'), 10, 1);
		add_filter( 'woocommerce_layered_nav_term_html', array($this, 'checkAttributeLayerdFilter'), 10, 4);

		if($this->get_option('categoryVariationSwatchesEnable')) {
			$hook = $this->get_option('categoryVariationSwatchesHook');
			$priority = $this->get_option('categoryVariationSwatchesHookPosition');
			add_action( $hook, array($this, 'displayCategoryVariationSwatches'), $priority);
		}

		if($this->get_option('categoryAttributeImagesEnable')) {
			$hook = $this->get_option('categoryAttributeImagesHook');
			$priority = $this->get_option('categoryAttributeImagesHookPosition');
			add_action( $hook, array($this, 'displaycategoryAttributeImages'), $priority);
		}		

		if($this->get_option('singleProductAttributeImagesEnable')) {
			$hook = $this->get_option('singleProductAttributeImagesHook');
			$priority = $this->get_option('singleProductAttributeImagesHookPosition');
			add_action( $hook, array($this, 'displaySingleProductAttributeImages'), $priority);
		}		
    }
	
	public function modify_attribute_template( $located, $template_name)
	{
		global $post;

		if($this->get_option('attributeTableEnable') && ('single-product/product-attributes.php' === $template_name) && !class_exists('WooCommerce_Group_Attributes')) {
			return  __DIR__  . '/partials/woocommerce-attribute-table-images.php';
		}

		if($this->get_option('variationsEnable') && ('single-product/add-to-cart/variable.php' === $template_name)) {
			return  __DIR__  . '/partials/woocommerce-variable-images.php';
		}

		return $located;
	}

	public function checkAttributeImage($attributeValue, $attributeID, $returnImage = true) 
	{
		$image = "";

		// if(!$this->get_option('attributeTableEnable')) {
		// 	return $attributeValue;
		// }

		if(!$returnImage) {
			return $attributeValue;
		}

		if(isset($this->term_data[$attributeID])) {
			$data = $this->term_data[$attributeID];
			if(isset($data['thumbnail']) && 
				!empty($data['thumbnail'])) {


				$imageSize = $this->get_option('attributeImageSize');	
				$attribute_value_image_src = wp_get_attachment_image_src($data['thumbnail'], $imageSize);
				
				if($this->get_option('showTooltip')) {
					$image = '<div class="woocommerce-attribute-image"><div data-attribute-tooltip title="' . $attributeValue . '" ><img src="' . $attribute_value_image_src[0] . '" alt="' . $attributeValue . '"></div>';
				} else {
					$image = '<div class="woocommerce-attribute-image"><img src="' . $attribute_value_image_src[0] . '" alt="' . $attributeValue . '">';
				}

				if($this->get_option('showTextBelow')) {
					$image .= '<div class="woocommerce-attribute-image-desc">' . $attributeValue . '</div>';
				}

				$image .= '</div>';
			} elseif(isset($data['color']) && 
				!empty($data['color'])) {

				if($this->get_option('showTooltip')) {
					$image = 
					'<div class="woocommerce-attribute-image">
						<div data-attribute-tooltip title="' . $attributeValue . '">
							<div class="woocommerce-attribute-image-color" style="background-color: ' . esc_url($data['color']) . '">'. $attributeValue . '</div>
						</div>';
				} else {
					$image = '<div class="woocommerce-attribute-image">
					<div class="woocommerce-attribute-image-color" title="'. $attributeValue . '" style="background-color: ' . esc_url($data['color']) . '">'. $attributeValue . '</div>';
				}

				if($this->get_option('showTextBelow')) {
					$image .= '<div class="woocommerce-attribute-image-desc">' . $attributeValue . '</div>';
				}

				$image .= '</div>';
				// $image = $attributeValue;
			} else {
				// $image = $attributeValue;
			}
		} else {
			// $image = $attributeValue;
		}

		return $image;
	}

	public function checkAttributeNameImage($attributeValue, $attributeID, $returnImage = true) 
	{
		$image = "";

		if(!$returnImage) {
			return $attributeValue;
		}

		if(isset($this->term_data[$attributeID])) {
			$data = $this->term_data[$attributeID];
			if(isset($data['thumbnail']) && 
				!empty($data['thumbnail'])) {


				$imageSize = $this->get_option('attributeNameImageSize');	
				$attribute_value_image_src = wp_get_attachment_image_src($data['thumbnail'], $imageSize);
				
				if($this->get_option('showTooltip')) {
					$image = '<div class="woocommerce-attribute-name-image"><div data-attribute-tooltip title="' . $attributeValue . '" ><img src="' . $attribute_value_image_src[0] . '" alt="' . $attributeValue . '"></div></div>';
				} else {
					$image = '<div class="woocommerce-attribute-name-image"><img src="' . $attribute_value_image_src[0] . '" alt="' . $attributeValue . '"></div>';
				}
			} elseif(isset($data['color']) && 
				!empty($data['color'])) {

				if($this->get_option('showTooltip')) {
					$image = '<div class="woocommerce-attribute-name-image"><div data-attribute-tooltip title="' . $attributeValue . '" ><div class="woocommerce-attribute-image-color" style="background-color: ' . esc_url($data['color']) . '">'. $attributeValue . '</div></div></div>';
				} else {
					$image = '<div class="woocommerce-attribute-name-image"><div class="woocommerce-attribute-image-color" title="'. $attributeValue . '" style="background-color: ' . esc_url($data['color']) . '">'. $attributeValue . '</div></div>';
				}
			} else {
				$image = $attributeValue;
			}
		} else {
			$image = $attributeValue;
		}

		return $image;
	}

	public function checkAttributeDescription($attribute_value) 
	{
		$description = "";
		if(isset($attribute_value->description) && !empty($attribute_value->description)) {
			$description = $attribute_value->description;
		}
		return wpautop($description);
	}

	public function checkAttributeLayerdFilter($term_html, $term, $link, $count) 
	{
		if(!$this->get_option('attributeFilterWidgetEnable')) {
			return $term_html;
		}

		$attributeID = $term->term_id;
		$attributeValue = $term->name;

		if(isset($this->term_data[$attributeID])) {
			$data = $this->term_data[$attributeID];
			if(isset($data['thumbnail']) && 
				!empty($data['thumbnail'])) {


				$imageSize = $this->get_option('attributeImageSize');	
				$attribute_value_image_src = wp_get_attachment_image_src($data['thumbnail'], $imageSize);
				
				if($this->get_option('showTooltip')) {
					$term_html = '<a href="' . $link . '" class="woocommerce-attribute-image"><div data-attribute-tooltip title="' . $attributeValue . '" ><img src="' . $attribute_value_image_src[0] . '" alt="' . $attributeValue . '"></div>';
				} else {
					$term_html = '<a href="' . $link . '" class="woocommerce-attribute-image"><img src="' . $attribute_value_image_src[0] . '" alt="' . $attributeValue . '">';
				}

				if($this->get_option('showTextBelow')) {
					$term_html .= '<div class="woocommerce-attribute-image-desc">' . $attributeValue . '</div>';
				}

				$term_html .= '</a>';
			} elseif(isset($data['color']) && 
				!empty($data['color'])) {
				
				if($this->get_option('showTooltip')) {
					$term_html = '<a href="' . $link . '" class="woocommerce-attribute-image"><div data-attribute-tooltip title="' . $attributeValue . '" ><div class="woocommerce-attribute-image-color"  style="background-color: ' . esc_url($data['color']) . '">'. $attributeValue . '</div></div>';
				} else {
					$term_html = '<a href="' . $link . '" class="woocommerce-attribute-image"><div class="woocommerce-attribute-image-color" title="'. $attributeValue . '" style="background-color: ' . esc_url($data['color']) . '">'. $attributeValue . '</div>';
				}

				if($this->get_option('showTextBelow')) {
					$term_html .= '<div class="woocommerce-attribute-image-desc">' . $attributeValue . '</div>';
				}

				$term_html .= '</a>';
			} else {

			}
		}

		return $term_html;
	}

	public function attributes_shortcode($atts)
	{
		$html = "";

		$args = shortcode_atts( array(
	        'product' => '',
	        'attributes' => '',
	    ), $atts );

		$product_id = absint($args['product']);
		if(empty($product_id)) {
			global $product;
		} else {
			 $product = wc_get_product($product_id);
		}	   

		if(!is_object($product)) {
			return false;
		}

		$attributes = $args['attributes'];
		if(empty($attributes)) {
			return false;
		}

		$attributes = explode(',', $attributes);

		$images = array();

		foreach ($attributes as $attribute) {

			$string = "";
			if (strpos($attribute, 'pa_') == FALSE){
				$attribute = 'pa_' . $attribute;
			}

			$taxonomy = get_taxonomy($attribute);
			if(!$taxonomy) {
				continue;
			}
			
			// Get terms if this is a taxonomy - ordered. We need the names too.
			$terms = wc_get_product_terms( $product->get_id(), $attribute, array(
				'fields' => 'all',
			) );

			if(empty($terms)) {
				continue;
			}

			$string .= '<div class="woocommerce-attribute-images-shortcode-attribute woocommerce-attribute-images-shortcode-attribute-' . $attribute . '">';

			foreach ( $terms as $term ) {

				$image = apply_filters('woocommerce_attribute_value_image', esc_html( $term->name ), $term->term_id, true);
				if ( !empty($image)) {
					if ( $taxonomy->public ) {
						$string .= '<a href="' . esc_url( get_term_link( $term->term_id, $attribute ) ) . '" data-attribute-value="' . $term->slug . '" data-attribute-name="' . $attribute_name . '">' . $image . '</a>';
					} else {
						$string .= '<span data-attribute-value="' . $term->slug . '" data-attribute-name="' . $attribute_name . '">' . $image . '</span>';	
					}
					
				}
			}
			$string .= '<div class="clear"></div></div>';
			$images[] = $string;
		}

		if(!empty($images)) {

			$html .= '<div class="woocommerce-attribute-images-shortcode woocommerce-attribute-images-variable-options">';
				$html .= implode(' ', $images);
			$html .= '<div class="clear"></div></div>';
		}
		
		return $html;
	}

	public function displayCategoryVariationSwatches()
	{
		global $product;

		$attributeVariableProduct = $product;

		// Ensure visibility.
		if ( empty( $attributeVariableProduct ) || ! $attributeVariableProduct->is_visible() ) {
			return;
		}

		if(function_exists('run_woocommerce_single_variations') && $this->get_option('singleVariationsSupport')) {
			if($attributeVariableProduct->is_type( 'variation') ) {
				$attributeVariableProduct = wc_get_product($attributeVariableProduct->get_parent_id());
			}
		}
		
		if(!$attributeVariableProduct->is_type( 'variable' )) {
			return;
		}

		$available_variations = $attributeVariableProduct->get_available_variations();
		if(empty($available_variations)) {
			return;
		}

		$attributes = $attributeVariableProduct->get_variation_attributes();

		echo '<div class="woocommerce-attribute-images-category-variation-swatches">';

			foreach ( $attributes as $attribute_name => $options ) {

				$selected = isset( $_REQUEST[ 'attribute_' . sanitize_title( $attribute_name ) ] ) ? wc_clean( stripslashes( urldecode( $_REQUEST[ 'attribute_' . sanitize_title( $attribute_name ) ] ) ) ) : $attributeVariableProduct->get_variation_default_attribute( $attribute_name );

				$images = array();

				if ( $attributeVariableProduct && taxonomy_exists( $attribute_name ) ) {
					// Get terms if this is a taxonomy - ordered. We need the names too.
					$terms = wc_get_product_terms( $attributeVariableProduct->get_id(), $attribute_name, array(
						'fields' => 'all',
					) );

					foreach ( $terms as $term ) {

						foreach ($available_variations as $available_variation) {
							$available_variation_attributes = $available_variation['attributes'];
							if(empty($available_variation_attributes)) {
								continue;
							}
							
							$variation = false;
							foreach ($available_variation_attributes as $available_variation_attribute_key => $available_variation_attribute_value) {
								if($available_variation_attribute_key == "attribute_" . $attribute_name && $available_variation_attribute_value == $term->slug) {
									$variation = new WC_Product_Variation( $available_variation['variation_id'] );

									$variationId = $variation->get_id();
									$variationPrice = htmlspecialchars( $variation->get_price_html() );
									$variationLink = $variation->get_permalink();
									$variationImage = "";
									$variationImageID = $variation->get_image_id();
									if(!empty($variationImageID)) {
										$variationImage = wp_get_attachment_image_src($variationImageID, 'woocommerce_thumbnail')[0];
									}
									break 2;
								}
							}
						}
						

						$image = apply_filters('woocommerce_attribute_value_image', esc_html( $term->name ), $term->term_id, true);
						if ( in_array( $term->slug, $options, true ) && !empty($image) && $term->name != $image) {
							$class = '';
							if(!$variation) {
								$class = ' woocommerce-attribute-images-variable-option-disabled';
							}
							
							if($variation && !$variation->is_in_stock()) {
								$class = ' woocommerce-attribute-images-variable-option-disabled';
							}

							$images[] = '<a href="#" class="woocommerce-attribute-images-variable-option ' . $class . '" data-variation-id="' . $variationId . '" data-variation-image="' . $variationImage . '" data-variation-price="' . $variationPrice . '" data-variation-link="' . $variationLink . '" data-attribute-value="' . $term->slug . '" data-attribute-name="' . $attribute_name . '">' . $image . '</a>';
						}
					}
				}

				if(!empty($images) && count($images) == count($options)) {

					ksort($images);

					echo '<div class="woocommerce-attribute-images-variable-options">';
						echo implode(' ', $images);
					echo '<div class="clear"></div></div>';
					echo '<div style="display: none;">';
				}

				if(!empty($images) && count($images) == count($options)) {
					echo '</div>';
				}
			}
		echo '</div>';
	}

	public function displaycategoryAttributeImages()
	{
		global $product;

		// Ensure visibility.
		if ( empty( $product ) || ! $product->is_visible() ) {
			return;
		}

		$attributes = $this->get_option('categoryAttributeImagesAttributes');
		if(empty($attributes)) {
			return;
		}

		$attributes = array_combine($attributes, $attributes);
		$variationAttributes = array();
		if($product->is_type( 'variable' )) {
			$variationAttributes = $product->get_variation_attributes();
		}

		$attributes = array_diff_key($attributes, $variationAttributes);

		$images = array();
		foreach ($attributes as $attribute) {

			$terms = wc_get_product_terms( $product->get_id(), $attribute, array(
				'fields' => 'all',
			) );

			if(empty($terms)) {
				continue;
			}

			foreach($terms as $term) {
				$images[] = apply_filters('woocommerce_attribute_value_image', esc_html( $term->name ), $term->term_id, true);
			}
		}

		if(empty($images)) {
			return;
		}

		echo '<div class="woocommerce-attribute-images-category-attribute-images">';

		ksort($images);

			echo '<div class="woocommerce-attribute-images-attributes">';
				echo implode(' ', $images);
				echo '<div class="clear"></div>';
			echo '</div>';			

		echo '</div>';
	}

	public function displaySingleProductAttributeImages()
	{
		global $product;

		if ( empty( $product ) || ! $product->is_visible() ) {
			return;
		}

		$attributes = $this->get_option('singleProductAttributeImagesAttributes');
		if(empty($attributes)) {
			return;
		}

		$images = array();
		foreach ($attributes as $attribute) {

			$taxonomy = get_taxonomy($attribute);
			if(!$taxonomy) {
				continue;
			}

			$terms = wc_get_product_terms( $product->get_id(), $attribute, array(
				'fields' => 'all',
			) );

			if(empty($terms)) {
				continue;
			}

			foreach($terms as $term) {

				$image = apply_filters('woocommerce_attribute_value_image', esc_html( $term->name ), $term->term_id, true);
				if(empty($image)) {
					continue;
				}

				if ( $taxonomy->public ) {
					$image = '<a href="' . esc_url( get_term_link( $term->term_id, $attribute ) ) . '" rel="tag">' . $image . '</a>';
				}

				$images[] = $image;
			}
		}

		if(empty($images)) {
			return;
		}

		echo '<div class="woocommerce-attribute-images-single-product-attribute-images">';

		ksort($images);

			echo '<div class="woocommerce-attribute-images-attributes">';
				echo implode(' ', $images);
				echo '<div class="clear"></div>';
			echo '</div>';			

		echo '</div>';
	}
	
}